@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
  * Copyright (C) 2010-2011  Université de Bordeaux 1
- * Copyright (C) 2010-2011  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010-2012  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -26,8 +26,8 @@ struct starpu_task_list sched_list;
 static pthread_cond_t sched_cond;
 static pthread_mutex_t sched_mutex;
 
-static void init_dummy_sched(struct starpu_machine_topology_s *topology,
-			struct starpu_sched_policy_s *policy)
+static void init_dummy_sched(struct starpu_machine_topology *topology,
+			struct starpu_sched_policy *policy)
 {
 	/* Create a linked-list of tasks and a condition variable to protect it */
 	starpu_task_list_init(&sched_list);
@@ -42,8 +42,8 @@ static void init_dummy_sched(struct starpu_machine_topology_s *topology,
 	FPRINTF(stderr, "Initialising Dummy scheduler\n");
 }
 
-static void deinit_dummy_sched(struct starpu_machine_topology_s *topology,
-				struct starpu_sched_policy_s *policy)
+static void deinit_dummy_sched(struct starpu_machine_topology *topology,
+				struct starpu_sched_policy *policy)
 {
 	STARPU_ASSERT(starpu_task_list_empty(&sched_list));
 
@@ -77,7 +77,8 @@ static struct starpu_task *pop_task_dummy(void)
 	return starpu_task_list_pop_back(&sched_list);
 }
 
-static struct starpu_sched_policy_s dummy_sched_policy = {
+static struct starpu_sched_policy dummy_sched_policy =
+{
 	.init_sched = init_dummy_sched,
 	.deinit_sched = deinit_dummy_sched,
 	.push_task = push_task_dummy,
@@ -88,7 +89,8 @@ static struct starpu_sched_policy_s dummy_sched_policy = {
 	.policy_description = "dummy scheduling strategy"
 };
 
-static struct starpu_conf conf = {
+static struct starpu_conf conf =
+{
 	.sched_policy_name = NULL,
 	.sched_policy = &dummy_sched_policy,
 	.ncpus = -1,
@@ -105,12 +107,12 @@ static void dummy_func(void *descr[] __attribute__ ((unused)), void *arg __attri
 {
 }
 
-static starpu_codelet dummy_codelet = 
+static struct starpu_codelet dummy_codelet =
 {
 	.where = STARPU_CPU|STARPU_CUDA|STARPU_OPENCL,
-	.cpu_func = dummy_func,
-	.cuda_func = dummy_func,
-        .opencl_func = dummy_func,
+	.cpu_funcs = {dummy_func, NULL},
+	.cuda_funcs = {dummy_func, NULL},
+        .opencl_funcs = {dummy_func, NULL},
 	.model = NULL,
 	.nbuffers = 0
 };
@@ -119,8 +121,12 @@ static starpu_codelet dummy_codelet =
 int main(int argc, char **argv)
 {
 	int ntasks = NTASKS;
+	int ret;
 
-	starpu_init(&conf);
+	ret = starpu_init(&conf);
+	if (ret == -ENODEV)
+		return 77;
+	STARPU_CHECK_RETURN_VALUE(ret, "starpu_init");
 
 #ifdef STARPU_SLOW_MACHINE
 	ntasks /= 100;
@@ -134,7 +140,8 @@ int main(int argc, char **argv)
 		task->cl = &dummy_codelet;
 		task->cl_arg = NULL;
 	
-		starpu_task_submit(task);
+		ret = starpu_task_submit(task);
+		STARPU_CHECK_RETURN_VALUE(ret, "starpu_task_submit");
 	}
 
 	starpu_task_wait_for_all();
