@@ -1,7 +1,7 @@
 /* StarPU --- Runtime system for heterogeneous multicore architectures.
  *
- * Copyright (C) 2010  Université de Bordeaux 1
- * Copyright (C) 2010  Centre National de la Recherche Scientifique
+ * Copyright (C) 2010, 2012  Université de Bordeaux 1
+ * Copyright (C) 2010, 2011  Centre National de la Recherche Scientifique
  *
  * StarPU is free software; you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -24,15 +24,15 @@
 #include <core/sched_policy.h>
 #include <core/dependencies/data_concurrency.h>
 
-/* We assume that j->sync_mutex is taken by the caller */
-void _starpu_notify_dependencies(struct starpu_job_s *j)
+/* We assume that the job will not disappear under our hands */
+void _starpu_notify_dependencies(struct _starpu_job *j)
 {
 	STARPU_ASSERT(j);
 	STARPU_ASSERT(j->task);
 
 	/* unlock tasks depending on that task */
 	_starpu_notify_task_dependencies(j);
-	
+
 	/* unlock tags depending on that task */
 	if (j->task->use_tag)
 		_starpu_notify_tag_dependencies(j->tag);
